#include <math.h>

typedef struct
{
	float x, y, z, thx, thy, thz, cx, cy, cz, sx, sy, sz;
} Camera;

typedef struct
{
	float x, y;
	int ix, iy;
	int beyond;
} Point2D;

typedef struct
{
	float x, y, z;
} Point3D;

void CameraChanged(Camera *C)
{
	C->cx = cos(C->thx);
	C->cy = cos(C->thy);
	C->cz = cos(C->thz);
	C->sx = sin(C->thx);
	C->sy = sin(C->thy);
	C->sz = sin(C->thz);
}

void TransformPoint3DToPoint2D(Point3D *p3D, Camera *C, Point2D *p2D, int w, int h)
{
	float x, y, z, dx, dy, dz, ez;

	if ((p3D == NULL) || (C == NULL) || (p2D == NULL))
	{
		return;
	}

	x = p3D->x - C->x;
	y = p3D->y - C->y;
	z = p3D->z - C->z;
	ez = 0.17632698;


	dz = C->cx * (C->cy * z + C->sy * (C->sz * y + C->cz * x)) - C->sx * (C->cz * y - C->sz * x);
	p2D->beyond = dz < 0.0;

	dx = C->cy * (C->sz * y + C->cz * x) - C->sy * z;
	p2D->x = (ez / dz) * dx;
	p2D->ix = (int)round(((w / 2.0 ) * p2D->x) + (w / 2.0));
	if (p2D->ix < 0)
	{
		p2D->ix = 0;
	}
	else if (p2D->ix >= w)
	{
		p2D->ix = w - 1;
	}

	dy = C->sx * (C->cy * z + C->sy * (C->sz * y + C->cz * x)) + C->cx * (C->cz * y - C->sz * x);
	p2D->y = (ez / dz) * dy;
	p2D->iy = (int)round(((h / 2.0) * p2D->y) + (h / 2.0));
	if (p2D->iy < 0)
	{
		p2D->iy = 0;
	}
	else if (p2D->iy >= h)
	{
		p2D->iy = h - 1;
	}
}
